@@ -10,6 +10,7 @@ import os
 import math
 import logging
 import mmap
+import time
 import multiprocessing as mp
 import argparse
 # import datetime
@@ -36,7 +37,8 @@ class readSam(object):
         """
         self._r1 = sam_r1
         self._r2 = sam_r2
-        self._project, self._seq, self._cds_seq, self._tile_map, self._region_map, self._samples, self._var = help_functions.parse_json(param)
+        self._param = param
+        self._project, self._seq, self._cds_seq, self._tile_map, self._region_map, self._samples, self._var, self._relations = help_functions.parse_json(param)
 
         self._qual = self._var["posteriorThreshold"]
         min_cover = self._var["minCover"]
@@ -58,7 +60,7 @@ class readSam(object):
 
         self._sample_condition = self._sample_info["Condition"].values[0]
         self._sample_tp = self._sample_info["Time point"].values[0]
-        self._sample_rep = self._sample_info["Replicate"].values[0]
+        self._sample_rep = int(self._sample_info["Replicate"].values[0])
 
         self._seq_lookup = help_functions.build_lookup(self._cds_start.values.item(), self._seq.cds_end.values.item(), self._cds_seq)
 
@@ -113,224 +115,101 @@ class readSam(object):
         self._track_reads = self._track_reads.set_index("pos")
         self._track_reads = self._track_reads.fillna(0)
 
-    def _merged_main(self):
+    def adjust_er(self, wt_override=False):
         """
-        Read two sam files at the same time, store mutations that passed filter
+        Based on the wt samples given in the parameter sheet,
+        calculate new error rate for phred scores
+        this function is dependent on the tileseqMave package
+        please make sure the correct script is installed before running the pipeline
         """
-        read_pair = 0 # total pairs
-        un_map = 0 # total number of unmapped reads
-        read_nomut = 0 # read pairs that have no mutations
+        r_df = pd.DataFrame(self._relations)
+        phred_output = []
+        
+        if r_df.empty:
+            if not wt_override:
+                # no wt relationship is defined
+                self._mut_log.warning("No WT relationship! No error rate correction can be applied")
+                return phred_output
 
-        hgvs_output = {} # save all the hgvs in this pair of sam files
-        r1_pop_hgvs = {}  # save all the hgvs that are 2/3nt changes based only on R1
-        r2_pop_hgvs = {}  # save all the hgvs that are 2/3nt changes based only on R2
-        off_mut = {} # save all the positions that were mapped but outside of the tile
-        row = {} # create a dictionary to save all the reads information to pass on to locate_mut.py
-        all_df = []
-        final_pairs = 0
-        r1_popmut = 0
-        r2_popmut = 0
-        off_read = 0
+            else:  # treating EVERYTHING in the run as wt
+                self._mut_log.warning("WT OVERRIDE ON, TREATING EVERYTHING AS WT")
+                wt = pd.DataFrame({}, columns=["Condition 1", "Relationship", "Condition 2"])
+                wt["Condition 1"] = self._samples["Condition"]
+        else:
+            # find out wt sample ID for this sample
+            wt = r_df[r_df["Relationship"]=="is_wt_control_for"]
+            if wt.empty:
+                if not wt_override:
+                    # no wt relationship is defined
+                    self._mut_log.warning("No WT relationship! No error rate correction can be applied")
+                    return phred_output
 
-        chunkSize = 1500000 # number of characters in each chunk (you will need to adjust this)
-        chunk1 = deque([""]) #buffered lines from 1st file
-        chunk2 = deque([""]) #buffered lines from 2nd file
-        with open(self._r1, "r") as r1_f, open(self._r2, "r") as r2_f:
-            while chunk1 and chunk2:
-                line_r1 = chunk1.popleft()
-                if not chunk1:
-                    line_r1,*more = (line_r1+r1_f.read(chunkSize)).split("\n")
-                    chunk1.extend(more)
-                line_r2 = chunk2.popleft()
-                if not chunk2:
-                    line_r2,*more = (line_r2+r2_f.read(chunkSize)).split("\n")
-                    chunk2.extend(more)
+                else:  # treating EVERYTHING in the run as wt
+                    self._mut_log.warning("WT OVERRIDE ON, TREATING EVERYTHING AS WT")
+                    wt["Condition 1"] = self._samples["Condtion"]
+        wt_id = ""
 
-                if line_r1.startswith("@") or line_r2.startswith("@"): # skip header lines
-                    continue
+        if self._sample_condition in wt["Condition 1"].tolist():
+            if self._sample_rep == 1:
+                # this sample is wt 
+                # run calibratePhred 
+                # we only need 1 wt calibration for each tile (r1 and r2)
+                # we are always using the first rep 
+                wt_id = self._sample_id
+            else:
+                # this is rep 2 of the wt
+                # find corresponding rep 1 of the wt 
+                wt_id = self._samples[(self._samples["Condition"] == self._sample_condition) & (self._samples["Tile ID"] == self._sample_tile) & (self._samples["Replicate"] == 1)]["Sample ID"].values[0]
 
-                line_r1 = line_r1.split()
-                line_r2 = line_r2.split()
-                if len(line_r1) < 11 or len(line_r2) < 11:
-                    # the read has no sequence
-                    self._mut_log.warning(line_r1)
-                    self._mut_log.warning(line_r2)
-                    self._mut_log.warning("Missing fields in read!")
-                    continue
+        elif self._sample_condition in wt["Condition 2"].tolist():
+            # get corresponding wt 
+            wt_name = wt[wt["Condition 2"] == self._sample_condition]["Condition 1"].values[0]
+            # get wt sample for this tile and rep 1
+            wt_id = self._samples[(self._samples["Condition"] == wt_name) & (self._samples["Tile ID"] == self._sample_tile) & (self._samples["Replicate"] == 1)]["Sample ID"].values[0]
+        
+        # check if calibrated
+        self._mut_log.info(f"wt sample used to adjust phred scores: {wt_id}")
+        phred_output_r1 = os.path.join(self._output_counts_dir, f"{wt_id}_T{self._sample_tile}_R1_calibrate_phred.csv")
+        phred_output_r2 = os.path.join(self._output_counts_dir, f"{wt_id}_T{self._sample_tile}_R2_calibrate_phred.csv")
+        if not os.path.isfile(phred_output_r1):
+            # create an empty file as place holder 
+            with open(phred_output_r1, 'w') as fp:
+                pass
+            log_f = os.path.join(self._output_counts_dir, f"{wt_id}_R1_phred.log")
+            cmd_r1 = f"calibratePhred.R {self._r1} -p {self._param} -o {phred_output_r1} -l {log_f} --silent --cores {self._cores}"
+            os.system(cmd_r1)
+        if not os.path.isfile(phred_output_r2):
+            # create an empty file as place holder 
+            with open(phred_output_r2, 'w') as fp:
+                pass
+            log_f = os.path.join(self._output_counts_dir, f"{wt_id}_R2_phred.log")
+            cmd_r2 = f"calibratePhred.R {self._r2} -p {self._param} -o {phred_output_r2} -l {log_f} --silent --cores {self._cores}"
+            os.system(cmd_r2)
+        
+        # check if both file has something in there
+        # if they are empty, wait for them to finish (they might be running in other jobs 
 
-                # count how many read pairs in this pair of sam files
-                read_pair += 1
-                mapped_name_r1 = line_r1[2]
-                mapped_name_r2 = line_r2[2]
-                if mapped_name_r1 == "*" or mapped_name_r2 == "*": # if one of the read didn't map to ref
-                    un_map +=1
-                    continue
+        t0 = time.time()  # check for timeout
+        while os.stat(phred_output_r1).st_size == 0:
+            os.system("sleep 300")
+            self._mut_log.warning("phred file (R1) found, but empty.. Waiting for the job to finish...")
+            t1 = time.time()
+            total = float(t1-t0)
+            if total > 10800:
+                raise TimeoutError(f"Wating for Phred file {adjustthred[0]} timeout")
+        t0 = time.time()
+        while os.stat(phred_output_r2).st_size == 0:
+            os.system("sleep 300")
+            self._mut_log.warning("phred file (R2) found, but empty.. Waiting for the job to finish...")
+            t1 = time.time()
+            total = float(t1-t0)
+            if total > 10800:
+                raise TimeoutError(f"Wating for Phred file {adjustthred[0]} timeout")
+        time.sleep(30)
+        self._mut_log.info(f"Adjusted thred files generated: {phred_output_r1}, {phred_output_r2}")
+        return [phred_output_r1, phred_output_r2]
 
-                # check if read ID mapped
-                read_name_r1 = line_r1[0]
-                read_name_r2 = line_r2[0]
-                if read_name_r1 != read_name_r2:
-                    print(read_name_r1)
-                    print(read_name_r2)
-                    self._mut_log.error("Read pair IDs did not map, please check fastq files")
-                    exit(1)
-
-                # get starting position for r1 and r2
-                pos_start_r1 = line_r1[3]
-                pos_start_r2 = line_r2[3]
-                # record reads that are not mapped to this tile
-                # this is defined as if the read covers at least min_map percent of the tile
-                # the default min_map is 70%
-                # we also assume that the read len is ALWAYS 150
-                r1_end = int(pos_start_r1) + 150
-                # r1_end must cover from start of the tile to 60% of the tile
-                if ((r1_end - int(self._cds_start)) < (int(self._tile_begins) + int(self._min_map_len))) or \
-                        ((int(pos_start_r2) - int(self._cds_start)) > (int(self._tile_ends) - int(self._min_map_len))):
-                    off_read += 1
-                    continue
-
-                # get CIGAR string
-                CIGAR_r1 = line_r1[5]
-                # get sequence
-                seq_r1 = line_r1[9]
-                # get quality str
-                quality_r1 = line_r1[10]
-
-                CIGAR_r2 = line_r2[5]
-                seq_r2 = line_r2[9]
-                quality_r2 = line_r2[10]
-
-                # get mdz str
-                mdz_r1 = [i for i in line_r1 if "MD:Z:" in i]
-                mdz_r2 = [i for i in line_r2 if "MD:Z:" in i]
-
-                if len(mdz_r1) != 0:
-                    mdz_r1 = mdz_r1[0].split(":")[-1]
-                else:
-                    mdz_r1 = ""
-
-                if len(mdz_r2) != 0:
-                    mdz_r2 = mdz_r2[0].split(":")[-1]
-                else:
-                    mdz_r2 = ""
-
-                if ((not re.search('[a-zA-Z]', mdz_r1)) and ("I" not in CIGAR_r1) and ("D" not in CIGAR_r1)) and \
-                        ((not re.search('[a-zA-Z]', mdz_r2)) and ("I" not in CIGAR_r2) and ("D" not in CIGAR_r2)):
-                    # if MDZ string only contains numbers
-                    # and no insertions shown in CIGAR string
-                    # means there is no mutation in this read
-                    # if both reads have no mutations in them, skip this pair
-                    read_nomut +=1
-                    # remove reads that have no mutations in MDZ
-                    continue
-
-                # make the reads in the format of a dictionary
-                # columns=["mapped_name", "pos_start", "qual", "CIGAR", "mdz","seq"])
-
-                #row["mapped_name_r1"] = mapped_name_r1
-                row["pos_start_r1"] = pos_start_r1
-                row["qual_r1"] = quality_r1
-                row["cigar_r1"] = CIGAR_r1
-                row["mdz_r1"] = mdz_r1
-                row["seq_r1"] = seq_r1
-
-                #row["mapped_name_r2"] = mapped_name_r2
-                row["pos_start_r2"] = pos_start_r2
-                row["qual_r2"] = quality_r2
-                row["cigar_r2"] = CIGAR_r2
-                row["mdz_r2"] = mdz_r2
-                row["seq_r2"] = seq_r2
-
-                # pass this dictionary to locate mut
-                # mut = locate_mut_main()
-                # add mutation to mut list
-                mut_parser = locate_mut.MutParser(row, self._seq, self._cds_seq, self._seq_lookup,
-                                                  self._tile_begins, self._tile_ends, self._qual,
-                                                  self._mut_log, self._mutrate, self._base, self._posteriorQC)
-                hgvs, outside_mut, all_posterior, hgvs_r1_clusters, hgvs_r2_clusters, track_df = mut_parser._main()
-                track_df = track_df.set_index("pos")
-                track_df = track_df[track_df.index.isin(self._track_reads.index)]
-                # add track df to track summary 
-                track_all = pd.concat([self._track_reads, track_df], axis=1).fillna(0)
-                self._track_reads = track_all.groupby(by=track_all.columns, axis=1).sum()
-                if len(hgvs) != 0:
-                    final_pairs += 1
-                    if hgvs_output.get(hgvs, -1) != -1:
-                        hgvs_output[hgvs] += 1
-                    else:
-                        hgvs_output[hgvs] = 1
-
-                if len(hgvs_r1_clusters) != 0:
-                    r1_popmut += 1
-                    if hgvs_r1_clusters in r1_pop_hgvs:
-                        r1_pop_hgvs[hgvs_r1_clusters] += 1
-                    else:
-                        r1_pop_hgvs[hgvs_r1_clusters] = 1
-
-                if len(hgvs_r2_clusters) != 0:
-                    r2_popmut += 1
-                    if hgvs_r2_clusters in r2_pop_hgvs:
-                        r2_pop_hgvs[hgvs_r2_clusters] += 1
-                    else:
-                        r2_pop_hgvs[hgvs_r2_clusters] = 1
-
-                if outside_mut != []:
-                    outside_mut = list(set(outside_mut))
-                    for i in outside_mut:
-                        if not (i in off_mut):
-                            off_mut[i] = 1
-                if not all_posterior.empty:
-                    all_df.append(all_posterior)
-
-        output_csv = open(self._sample_counts_f, "a")
-        output_csv.write(f"#Raw read depth:{read_pair}\n")
-        output_csv.write(
-            f"#Number of read pairs without mutations:{read_nomut}\n#Number of read pairs did not map to gene:{un_map}\n#Mutation positions outside of the tile:{off_mut}\n#Number of reads outside of the tile:{off_read}\n")
-        output_csv.write(f"#Final read-depth:{read_pair - un_map - off_read}\n")
-        output_csv.write(f"#Total read pairs with mutations:{final_pairs}\n")
-        output_csv.write(
-            f"#Comment: Total read pairs with mutations = Read pairs with mutations that passed the posterior threshold\n#Comment: Final read-depth = raw read depth - reads didn't map to gene - reads mapped outside of the tile\n")
-
-        output_csv.close()
-
-        self._mut_log.info(f"Raw sequencing depth: {read_pair}")
-        self._mut_log.info(f"Number of reads without mutations:{read_nomut}")
-        self._mut_log.info(f"Final read-depth: {read_pair - un_map - off_read}")
-
-        # save read coverage to csv file 
-        cov_file = os.path.join(self._output_counts_dir, f"{self._sample_id}_coverage.csv")
-        self._track_reads.to_csv(cov_file)
-        # convert list to df with one col
-        hgvs_df = pd.DataFrame.from_dict(hgvs_output, orient="index")
-        hgvs_df = hgvs_df.reset_index()
-        if not hgvs_df.empty:
-            hgvs_df.columns = ["HGVS", "count"]
-        hgvs_df.to_csv(self._sample_counts_f, mode="a", index=False)
-        del hgvs_df
-
-        if self._posteriorQC:
-            r1_df = pd.DataFrame.from_dict(r1_pop_hgvs, orient="index")
-            r1_df = r1_df.reset_index()
-            if not r1_df.empty:
-                r1_df.columns = ["HGVS", "count"]
-            r1_df.to_csv(self._sample_counts_r1_f, mode="a", index=False)
-            del r1_f
-
-            r2_df = pd.DataFrame.from_dict(r2_pop_hgvs, orient="index")
-            r2_df = r2_df.reset_index()
-            if not r2_df.empty:
-                r2_df.columns = ["HGVS", "count"]
-            r2_df.to_csv(self._sample_counts_r2_f, mode="a", index=False)
-            del r2_f
-
-            self._mut_log.info(f"Reading posterior unfiltered dfs ...")
-            all_f = os.path.join(self._output_counts_dir, f"{self._sample_id}_posprob_all.csv")
-            for df in all_df:
-                df.to_csv(all_f, mode='a', header=False, index=False)
-            self._mut_log.info(f"Posterior df saved to files ... {all_f}")
-            del all_df
-
-    def multi_core(self):
+    def multi_core(self, adjusted_er):
         """
         Read two sam files at the same time, store mutations that passed filter
         """
@@ -446,7 +325,7 @@ class readSam(object):
                                               # self._tile_ends, self._qual, self._locate_log, self._mutrate)
             jobs.append(pool.apply_async(process_wrapper, (row, self._seq, self._cds_seq, self._seq_lookup, self._tile_begins,
                                                self._tile_ends, self._qual, self._mut_log, self._mutrate,
-                                                           self._base, self._posteriorQC)))
+                                                           self._base, self._posteriorQC, adjusted_er)))
             row = {} # flush
 
         r1_f.close()
@@ -554,52 +433,13 @@ class readSam(object):
             self._mut_log.info(f"Posterior df saved to files ... {all_f}")
             del all_df
 
+
 def process_wrapper(row, seq, cds_seq, seq_lookup, tile_begins, tile_ends, qual, locate_log, mutrate, base,
-                    posteriorQC):
+                    posteriorQC, adjusted_er):
     """
 
     """
     mut_parser = locate_mut.MutParser(row, seq, cds_seq, seq_lookup, tile_begins, tile_ends, qual, locate_log,
-                                      mutrate, base, posteriorQC)
+                                      mutrate, base, posteriorQC, adjusted_er)
     hgvs, outside_mut, all_df, hgvs_r1_clusters, hgvs_r2_clusters, track_df = mut_parser._main()
     return hgvs, outside_mut, all_df, hgvs_r1_clusters, hgvs_r2_clusters, track_df
-
-
-# if __name__ == "__main__":
-#
-#     parser = argparse.ArgumentParser(description='TileSeq mutation counts (for sam files)')
-#     parser.add_argument("-r1", "--read_1", help="sam file for R1", required=True)
-#     parser.add_argument("-r2", "--read_2", help="sam file for R2", required=True)
-#     parser.add_argument("-qual", "--quality", help="quality filter using posterior probability", default=0.99)
-#     parser.add_argument("-o", "--output", help="Output folder that contains the directory: ./sam_files", required = True)
-#     parser.add_argument("-log", "--log_level", help="Set log level: debug, info, warning, error, critical.", default = "debug")
-#     parser.add_argument("-mutlog", "--log_dir", help="Directory to save all the log files for each sample")
-#     parser.add_argument("-p", "--param", help="csv paramter file", required = True)
-#     parser.add_argument("-min", "--min_cover", help="Minimum percentage required to be covered by reads", default = 0.4)
-#
-#     args = parser.parse_args()
-#
-#     sam_r1 = args.read_1
-#     sam_r2 = args.read_2
-#     qual_filter = float(args.quality)
-#     log_dir = args.log_dir
-#     min_map = float(args.min_cover)
-#
-#     out = args.output
-#     param = args.param
-#
-#     # conver the csv file to json
-#     # csv2json = os.path.abspath("src/csv2json.R")
-#     param_json = param.replace(".csv", ".json")
-#     #convert = f"Rscript {csv2json} {param} -o {param_json} -l stdout"
-#     #os.system(convert)
-#
-#     # process the json file
-#     project, seq, cds_seq, tile_map, region_map, samples = help_functions.parse_json(param_json)
-#     # build lookup table
-#     lookup_df = help_functions.build_lookup(seq.cds_start.values.item(), seq.cds_end.values.item(), cds_seq)
-#
-#     # initialize the object
-#     MutCounts = readSam(sam_r1, sam_r2, seq, lookup_df, tile_map, region_map, samples, out, qual_filter, min_map, args.log_level, log_dir)
-#
-#     MutCounts._merged_main(seq, cds_seq)
