@@ -27,6 +27,7 @@ def merge_runs(input_dir1, input_dir2, map_df, output):
         merged_header = ""
         mut_count_f1 = os.path.join(input_dir1, f"counts_sample_{row['Sample ID_run1']}.csv")
         mut_count_f2 = os.path.join(input_dir2, f"counts_sample_{row['Sample ID_run2']}.csv")
+
         new_samples.append(f"{row['Sample ID_run1']}-{row['Sample ID_run2']}")
         # join headers
         with open(mut_count_f1) as file1, open(mut_count_f2) as file2:
@@ -63,7 +64,17 @@ def merge_runs(input_dir1, input_dir2, map_df, output):
         # save header and counts to file
         merge_counts.to_csv(output_f, mode="a", index=False)
         output_f.close()
+        
+        cov_f1 = os.path.join(input_dir1, f"coverage_{row['Sample ID_run1']}.csv")
+        cov_f2 = os.path.join(input_dir2, f"coverage_{row['Sample ID_run2']}.csv")
 
+        # read coverage files
+        # sum up two df
+        cov_d1 = pd.read_csv(cov_f1).set_index("pos")
+        cov_d2 = pd.read_csv(cov_f2).set_index("pos")
+        df_sum = cov_d1.add(cov_d2, fill_value=0)
+        output_cov = os.path.join(output, f"coverage_{row['Sample ID_run1']}-{row['Sample ID_run2']}.csv")
+        df_sum.to_csv(output_cov)
     return new_samples
 
 
@@ -75,12 +86,15 @@ def read_json(json_dict):
     condition_def = pd.DataFrame(json_dict["conditions"]["definitions"])
     # get nonselect conditions
     # get wt for nonselect
-    select_relation = condition_def.loc[condition_def["Relationship"].str.contains("is_selection_for")]
-    wt_relation = condition_def.loc[condition_def["Relationship"].str.contains("is_wt_control_for")]
-    select_relation = select_relation[["Condition 1", "Condition 2"]]
-    wt_relation = wt_relation[["Condition 1", "Condition 2"]]
-    relation = pd.merge(select_relation, wt_relation, how="left", left_on="Condition 2", right_on="Condition 2")
-    relation.columns = ["S", "NS", "WT"]
+    try:
+        select_relation = condition_def.loc[condition_def["Relationship"].str.contains("is_selection_for")]
+        wt_relation = condition_def.loc[condition_def["Relationship"].str.contains("is_wt_control_for")]
+        select_relation = select_relation[["Condition 1", "Condition 2"]]
+        wt_relation = wt_relation[["Condition 1", "Condition 2"]]
+        relation = pd.merge(select_relation, wt_relation, how="left", left_on="Condition 2", right_on="Condition 2")
+        relation.columns = ["S", "NS", "WT"]
+    except:
+        relation = pd.DataFrame({})
     samples_df = pd.DataFrame(samples)
     return relation, samples_df
 
